local addonName, ns = ...
local _G = _G
local MODULE_NAME = 'PlayerRep'
local addon = ns.addon
local module = addon:createModule(MODULE_NAME, addon.moduleTemplate, 'AceEvent-3.0')
local setfenv = _G.setfenv
setfenv(1, module)

local type, pairs = _G.type, _G.pairs
local pcall = _G.pcall
local assert = _G.assert

local min = _G.min

local GetFactionInfoByID = _G.GetFactionInfoByID
local GetNumFactions = _G.GetNumFactions
local GetFactionInfo, IsFactionInactive = _G.GetFactionInfo, _G.IsFactionInactive

local REP_DATA  = ns.REP_DATA
local REP_NAMES = ns.REP_NAMES

module.subModules = {}
local subModules = module.subModules

--[==[
-- Options
--]==]
--module.optionsTable.args.factionId = {
--    name = "Faction",
--    type = 'input',
--    validate = 'checkFaction',
--    get = 'getFaction',
--    set = 'setFaction',
--}
module.optionsTable.childGroups = 'tab'
module.optionsTable.args.factions = {
    name = "Factions",
    type = "multiselect",
    values = 'getFactions',
    get = 'getSelectedFactions',
    set = 'setSelectedFactions',
    order = 10,
}

--function module:getFaction(info)
--    return self:getOptions().factionId
--end

--function module:setFaction(info,id)
--    self:getOptions().factionId = id
--    self:Invalidate()
--end

function module:getSelectedFactions(info,id,...)
    --addon:Debug(info,id,...)
    return self:getOptions().factions[id] or false
end

function module:setSelectedFactions(info,id,value,...)
    --addon:Debug(info,id,value,...)
    self:getOptions().factions[id] = value
    if value then
        self:enableOrCreateSubModule(id)
    else
        self:disableSubModule(id)
    end
end

--local function isFaction(id)
--    if (not id) or (not type(id) == 'number') then
--       return "Not a number"
--    end
--    -- do a protected call, if it suceeds this is a valid faction
--    if pcall(function() GetFactionInfoByID(id) end) then
--        local name = GetFactionInfoByID(id)
--        --addon:Debug(id, name, "is a faction")
--        return true
--    else
--        --addon:Debug(id,"is no faction")
--        return "Not a valid faction"
--    end
--end

--function module:checkFaction(info, input)
--    return isFaction(input)
--end

module.defaults.factions = {}

--function _G.Grave01()
--    module:getFactions()
--end

local factionIDByName = {}
local knownFactions ={}

local function updateKnownFactions()
    local numFactions = GetNumFactions()
    for index = 1, numFactions do
        local name, description, standingID,
              barMin, barMax, barValue,
              atWarWith, canToggleAtWar,
              isHeader, isCollapsed,
              hasRep, isWatched,
              isChild = GetFactionInfo(index)
        local inactive = IsFactionInactive(index)
        knownFactions[name] = inactive and 1 or 0
    end
end

local function updateFactionIDByName()
    for factionID = 1, 2000 do
        local name, description, standingID,
              barMin, barMax, barValue,
              atWarWith, canToggleAtWar,
              isHeader, isCollapsed,
              hasRep, isWatched,
              isChild = GetFactionInfoByID(factionID)
        if knownFactions[name] then
            if not isHeader or hasRep then
                --addon:Debug("id=" .. factionID,
                --    ", name="..name,
                --    --", desc="..description,
                --    ", standingID="..standingID,
                --    ", war=".._G.tostring(atWarWith),canToggleAtWar,
                --    ", rep=[" .. _G.tostring(barMin) .. "," .. _G.tostring(barValue) .. "," .. _G.tostring(barMax) .. "]",
                --    ", header=".._G.tostring(isHeader),hasRep,isChild)
                factionIDByName[name] = factionID
            end
        end
    end
end

function module:getFactions()
    updateKnownFactions()
    updateFactionIDByName()
    local factions = {}
    for name,factionID in pairs(factionIDByName) do
        factions[factionID] = name
    end
    return factions
end

--[==[
-- End of options
--]==]

function module:createModule(name, template, ...)
    -- Note that AceAddon3:NewModule creates a shallow copy of the template.
    -- This is not sufficient, hence we need to make a deepcopy first
    local templateCopy = ns.deepCopy(template)
    local module = self:NewModule(name, templateCopy, 'AceHook-3.0',...)
    module.parent = self
    return module
end

--local oldOnEnable = module.OnEnable
function module:OnEnable()
    --oldOnEnable(module)
    for id, state in pairs(self:getOptions().factions) do
        if state then
            self:enableOrCreateSubModule(id)
        end
    end
end

function module:enableOrCreateSubModule(id)
    local subModule
    if subModules[id] then
        subModule = subModules[id]
        subModule:Enable()
    else
        local subModuleName = MODULE_NAME .. id
        addon:Debug("before createModule(" .. subModuleName..")")
        subModule = self:createModule(subModuleName, addon.playerRepBarTemplate, 'AceEvent-3.0')
        addon:Debug("after createModule(" .. subModuleName..")")
        subModule.factionID = id
        assert(subModule:GetName() == subModuleName)
        subModules[id] = subModule
        --subModule:OnInitialize()
        subModule:Enable()
    end
end

function module:disableSubModule(id)
    if subModules[id] then
        subModules[id]:Disable()
    end
end